@@ -331,59 +331,106 @@ class authres_status extends rcube_plugin
 
     public function get_authentication_status($headers, $show_statuses = 0, $uid = 0)
     {
+        $dmarcFailed = false;
         /* If dkimproxy did not find a signature, stop here
         */
         if (($results = ($headers->others['x-dkim-authentication-results'] ?? '')) && strpos($results, 'none') !== false) {
             $status = self::STATUS_NOSIG;
         } else {
-            if ($headers->others['authentication-results'] ?? null) {
+            $hasAuthenticationResultHeaders = (bool) isset($headers->others['authentication-results']);
+            if ($hasAuthenticationResultHeaders) {
                 $results = $this->rfc5451_extract_authresheader($headers->others['authentication-results']);
                 $status = 0;
                 $title = '';
 
                 foreach ($results as $result) {
-                    $status = $status | (isset(self::$RFC5451_authentication_results[$result['result']]) ? self::$RFC5451_authentication_results[$result['result']] : self::STATUS_FAIL);
-
+                    $status = $status | (self::$RFC5451_authentication_results[$result['result']] ?? self::STATUS_FAIL);
                     $title .= ($title ? '; ' : '') . $result['title'];
+
+                    if ($result['method'] === 'dmarc' && !$dmarcFailed) {
+                        $dmarcFailed = $result['result'] === 'fail';
+                    }
                 }
 
-                if ($status == self::STATUS_PASS) {
-                    /* Verify if its an author's domain signature or a third party
-                    */
-                    if (preg_match("/[@]([a-zA-Z0-9]+([.][a-zA-Z0-9]+)?\.[a-zA-Z]{2,4})/", $headers->from, $m)) {
+                if ($status === self::STATUS_PASS || $status === self::STATUS_THIRD) {
+                    // Extract emailaddress from From: header
+                    $authorEmail = '';
+                    $authorDomain = '';
+
+                    if (preg_match('/<([^>]+)>/', $headers->from, $m)) {
+                        $authorEmail = $m[1];
+                    }
+
+                    if (str_contains($authorEmail, '@')) {
+                        $authorDomain = explode('@', $authorEmail, 2)[1];
+                    }
+
+                    if ($authorDomain) {
                         $title = '';
-                        $authorDomain = $m[1];
                         $authorDomainFound = false;
 
+                        /* Verify if its an author's domain signature or a third party
+                        */
                         foreach ($results as $result) {
-                            if ($result['method'] == 'dkim' || $result['method'] == 'domainkeys') {
-                                if (is_array($result['props']) && isset($result['props']['header'])) {
-                                    $pvalue = '';
+                            if (
+                                !in_array($result['method'], ['auth', 'dkim', 'dmarc', 'domainkeys'])
+                                || !is_array($result['props'])
+                            ) {
+                                continue;
+                            }
 
-                                    // d is required, but still not always present
-                                    if (isset($result['props']['header']['d'])) {
-                                        $pvalue = $result['props']['header']['d'];
-                                    } elseif (isset($result['props']['header']['i'])) {
-                                        $pvalue = substr($result['props']['header']['i'], strpos($result['props']['header']['i'], '@') + 1);
-                                    }
+                            if ($result['method'] === 'auth') {
+                                $method_props = $result['props']['smtp'] ?? '';
+                            } else {
+                                $method_props = $result['props']['header'];
+                            }
 
-                                    if ($pvalue == $authorDomain || substr($authorDomain, -1 * strlen($pvalue)) == $pvalue) {
-                                        $authorDomainFound = true;
+                            if (!isset($method_props)) {
+                                continue;
+                            }
 
-                                        if ($status != self::STATUS_PASS) {
-                                            $status = self::STATUS_PASS;
-                                            $title = $result['title'];
-                                        } else {
-                                            $title.= ($title ? '; ' : '') . $result['title'];
-                                        }
-                                    } else {
-                                        if ($status == self::STATUS_THIRD) {
-                                            $title .= '; ' . $this->gettext('for') . ' ' . $pvalue . ' ' . $this->gettext('by') . ' ' . $result['title'];
-                                        } elseif (!$authorDomainFound) {
-                                            $status = self::STATUS_THIRD;
-                                            $title = $pvalue . ' ' . $this->gettext('by') . ' ' . $result['title'];
-                                        }
-                                    }
+                            $pvalue = ''; // pvalue refers to the definition in RFC-5451 (p10)
+
+                            if (isset($method_props['d'])) {
+                                // d is required, but still not always present
+                                $pvalue = $method_props['d'];
+                            } elseif (isset($method_props['i'])) {
+                                $pvalue = substr($method_props['i'], strpos($method_props['i'], '@') + 1);
+                            } elseif ($result['method'] === 'dmarc' && isset($method_props['from'])) {
+                                // from is used with dmarc results
+                                $pvalue = $method_props['from'];
+                            } elseif ($result['method'] === 'auth' && isset($method_props['mailfrom'])) {
+                                // mailfrom is used with smtp.auth results (RFC-4954)
+                                $pvalue = $method_props['mailfrom'];
+                            }
+
+                            $isAuthorValid = false;
+                            if ($result['method'] === 'auth') {
+                                $isAuthorValid = $pvalue === $authorEmail;
+                            } else {
+                                $isAuthorValid = $pvalue === $authorDomain;
+
+                                if (!$isAuthorValid) {
+                                    // Check if authorDomain is a subdomain of the signee
+                                    $isAuthorValid = str_ends_with($authorDomain, '.' . str_replace('@', '', $pvalue));
+                                }
+                            }
+
+                            if ($isAuthorValid) {
+                                $authorDomainFound = true;
+
+                                if ($status != self::STATUS_PASS) {
+                                    $status = self::STATUS_PASS;
+                                    $title = $result['title'];
+                                } else {
+                                    $title.= ($title ? '; ' : '') . $result['title'];
+                                }
+                            } else {
+                                if ($status == self::STATUS_THIRD) {
+                                    $title .= '; ' . $this->gettext('for') . ' ' . $pvalue . ' ' . $this->gettext('by') . ' ' . $result['title'];
+                                } elseif (!$authorDomainFound) {
+                                    $status = self::STATUS_THIRD;
+                                    $title = $pvalue . ' ' . $this->gettext('by') . ' ' . $result['title'];
                                 }
                             }
                         }
@@ -421,12 +468,12 @@ class authres_status extends rcube_plugin
                         }
                     }
                 }
-            } elseif ($headers->others['dkim-signature'] ?? null || $headers->others['domainkey-signature'] ?? null) {
+            } elseif (($headers->others['dkim-signature'] ?? null) || ($headers->others['domainkey-signature'] ?? null)) {
                 $status = 0;
 
                 if ($uid) {
                     $rcmail = rcmail::get_instance();
-                    if ($headers->others['dkim-signature'] ?? null && $rcmail->config->get('use_fallback_verifier')) {
+                    if (($headers->others['dkim-signature'] ?? null) && ($rcmail->config->get('use_fallback_verifier'))) {
                         if (!class_exists('Crypt_RSA')) {
                             $autoload = require __DIR__ . "/../../vendor/autoload.php";
                             $autoload->loadClass('Crypt_RSA'); // Preload for use in DKIM_Verify
@@ -436,7 +483,7 @@ class authres_status extends rcube_plugin
                           $dkimVerify = new DKIM_Verify($rcmail->storage->get_raw_body($uid));
                           $results = $dkimVerify->validate();
                         } catch(Exception $e) {
-                          $results = array();
+                          $results = [];
 
                           $status = self::STATUS_NOSIG;
                           $title = "Exception thrown by internal verifier: " . $e->getMessage();
@@ -478,12 +525,14 @@ class authres_status extends rcube_plugin
         } elseif ($status == self::STATUS_NORES) {
             $image = 'status_nores.png';
             $alt = 'noauthresults';
-        } elseif ($status == self::STATUS_PASS) {
+        } elseif (($status == self::STATUS_PASS) || ($status == self::STATUS_PASS + self::STATUS_NOSIG)) {
             $image = 'status_pass.png';
             $alt = 'signaturepass';
         } else {
             // at least one auth method was passed, show partial pass
-            if (($status & self::STATUS_PASS)) {
+            // unless DMARC failed, then we should fail all auth
+            // (see https://github.com/pimlie/authres_status/issues/68)
+            if (($status & self::STATUS_PASS) && !$dmarcFailed) {
                 $status = self::STATUS_PARS;
                 $image = 'status_partial_pass.png';
                 $alt = 'partialpass';
@@ -500,7 +549,7 @@ class authres_status extends rcube_plugin
         }
 
         if (!$show_statuses || ($show_statuses & $status)) {
-            return '<img src="plugins/authres_status/images/' . $image . '" alt="' . $alt . '" title="' . $this->gettext($alt) . htmlentities($title) . '" class="authres-status-img" /> ';
+            return '<img src="' . $this->urlbase . 'images/' . $image . '" alt="' . $alt . '" title="' . $this->gettext($alt) . htmlentities($title ?? '') . '" class="authres-status-img" /> ';
         }
 
         return '';
